Load <- function(var, exp, obs = NULL, sdates, lonmin = 0, lonmax = 360, 
                 latmin = -90, latmax = 90, nleadtime = NULL, nmember = NULL, 
                 leadtimemin = 1, leadtimemax = NULL, storefreq = 'monthly', 
                 sampleperiod = 1, output = 'areave', method = 'conservative', 
                 grid = NULL, maskmod = list(NULL, NULL, NULL, NULL, NULL, 
                 NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL), 
                 maskobs = list(NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                 NULL, NULL, NULL, NULL, NULL, NULL, NULL), 
                 configfile = NULL) {
  # TODO: Comprovar tots els TODO i totes els bugfix
  # TODO: COPIAR I ACTUALITZAR DOC I TOTS ELS COMENTARIS EN GENERAL
  # TODO: Acceptar que per les variables oceaniques 'si' posi nmembobs a 1 i per la resta a 5
  #  if (substr(var, 1, 2) == 'si') {
  #    nmembobs <- 1
  #  } else {
  #    nmembobs <- 5
  #  }

  # This function loads experimental data together with observational data at 
  # the corresponding dates from the specified experimental and observational 
  # datasets. It organizes the data in two matrices with dimensions:
  #  c(nexp/nmod, nmembers, nsdates, nleadtimes) for the experiments
  #  c(nobs/nreanalyses, nmembers, nsdates, nleadtimes) for the observations
  # up to
  #  c(nexp/nmod, nmembers, nsdates, nleadtimes, ndepth, nlat, nlon) 
  #  c(nobs/nreanalyses, nmembers, nsdates, nleadtimes, ndepth, nlat, nlon)
  # Only a specified variable and set of starting dates is loaded.
  # Once the two matrices are filled by calling this function, other functions
  # in the 2dverification package that require this data structure can be 
  # executed (e.g: Clim() to compute climatologies,  Ano() to compute 
  # anomalies, ...).
  #
  # Load() retrieves information about the location of the datasets from a 
  # configuration file that has to be filled in following the rules specified
  # in ?LoadConfigurationFile.
  #
  # The function Load() must return 4 a list which contains four components: 
  # $mod, $obs, $lat and $lon.
  # $mod is the matrix containing the experimental data.
  # $obs is the matrix containing the observational data.
  # $lat and $lon are the latitudes and longitudes of the model grid (0 if the
  # loaded variable is a global mean).
  #
  # The experimental data matrix will contain the values of a given variable for 
  # all the experiments the user wish to compare for a set of starting dates, 
  # lead-times, longitudes and latitudes interpolated to the common grid and
  # with the masks applied.
  # The observational data matrix will contain the observed values of the same
  # variable gathered from the specified observational datasets scanning the 
  # data repository. The observational data is chosen so as to date-correspond
  # the experimental data.
  #
  # The data matrices must be built with the dimensions in the following order 
  # and with the following lengths:
  # 1- The number of experimental datasets determined by the user (we need to 
  # store data of all the experiments in the same array) for the experimental 
  # matrix or the number of observational datasets available for validation in 
  # the observational matrix.
  # 2- The greatest number of members across all experiments.
  # 3- The number of starting dates determined by the user (we need to store 
  # data of each prediction of the model in each starting date).
  # 4- The greatest number of lead-times.
  # 5- The number of latitudes of the zone we want to consider.
  # 6- The number of longitudes of the zone we want to consider.
  #
  # Dimensions 5 and 6 are optional and their presence will depend on whether 
  # the variable loaded is 2-dimensional or an area average. 
  # In the case of an area average the dimensions of the matrix will be only the 
  # first 4.
  #
  # For example, at IC3, we have implemented Load() to select the experimental 
  # and observational datasets from which to load data, the variable to load, 
  # the range of starting dates, the range of lead-times, the frequency and step 
  # in which the data is loaded from the datasets, a common grid, a method of 
  # interpolation, a mask to disable certain experimental values, a mask to 
  # disable certain observational values and an output format:
  #   1) Time series of area-averaged variables over the specified domain. 
  #   2) Time series of meridional averages as a function of longitudes. 
  #   3) Time series of zonal averages as a function of latitudes.
  #   4) Time series of 2d fields.
  #
  # Furthermore, if the user loads a 2-dimensional variable, he/she can ask 
  # Load() to load it as area averages, as longitudinal averages in function of 
  # latitudes or as in latitudinal averages in function of longitudes. In these 
  # cases the 5th and/or 6th dimension will also disappear.
  #
  # The observational data matrix is built very similarly to the experimental 
  # matrix. The only differences are that the first dimension turns to be the
  # number of observational datasets and the length of the 2nd dimension turns  
  # to be the number of members of the observational datasets. Later, for each 
  # lead-time we will store observational data into it if possible.
  #
  # Any value that can't be filled (either in the experimental matrix or the 
  # observational matrix) should figure as an NA value.
  # 
  # Args:
  #   var: One of the following: 'tas', 'prlr', 'tos', 'g500', 'g200', 'ta50',
  #        'psl', 'hflsd', 'hfssd', 'rls', 'rss', 'rsds', 'uas', 'vas', 
  #        'tasmax_q90','tasmin_q90','prlr_q90','prlr_q10',
  #        'prlr_nbdaysq90_1979-2012','prlr_nbdaysq10_1979-2012',
  #        'prlr_nbdaysq90_1979-2005','prlr_nbdaysq10_1979-2005',
  #        'prlr_nbdaysq90_1981-2010','prlr_nbdaysq10_1981-2010',
  #        'tasmax_nbdaysq90_1979-2012','tasmin_nbdaysq10_1979-2012',
  #        'tasmax_nbdaysq90_1979-2005','tasmin_nbdaysq10_1979-2005',
  #        'sieN', 'sivN', 'siaS', 'sieS', 'sivS', 'sia_Arctic_Marginal_Seas',
  #        'sie_Arctic_Marginal_Seas', 'siv_Arctic_Marginal_Seas', [ and similarly
  #        'sia_' or 'sie_' or 'siv_' followed by 'Amundsen_Sea' or 
  #        'Antarctic_Atlantic_Sector' or 'Antarctic_Indian_Sector' or 
  #        'Antarctic_Ocean' or 'ArctOcn' or 'Arctic_Ocean' or 
  #        'Arctic_Ocean_North_Atlantic' or 'Atlantic_Ocean' or 'Baffin' or 
  #        'Baffin_Bay' or 'Baltic_Sea' or 'BarKara' or 'Barents_Sea' or
  #        'Beaufort_Chukchi_Sea' or 'Beaufort_Sea' or 'Bellingshausen_Sea' or 
  #        'Bering' or 'Bering_Strait' or 'CanArch' or 'Canadian_Waters' or 
  #        'Caspian_Sea' or 'Central_Arctic' or 'Chukchi_Sea' or 'East_Siberian_Sea'
  #         or 'Eastern_Central_Arctic' or 'Fram_Strait' or 'Global_Ocean' or 
  #        'Greenland_Sea' or 'Grnland' or 'Hudson' or 'Icelandic_Sea' or 
  #        'Indian_Ocean' or 'Indo_Pacific_Ocean' or 'Kara_Gate_Strait' or 
  #        'Kara_Sea' or 'LAT80N' or 'Labrador_Sea' or 
  #        'Laptev_East_Siberian_Chukchi_Seas' or 'Laptev_East_Siberian_Seas' or
  #        'Laptev_Sea' or 'Lincoln_Sea' or 'Mediterranean_Sea' or 'Nares_Strait'
  #        'Nordic_Barents_Seas' or 'Nordic_Seas' or 'NorthWest_Passage' or
  #        'North_Atlantic_Arctic' or 'North_Atlantic_Ocean' or 
  #        'North_Hemisphere_Ocean' or 'Northern_Hemisphere' or 'Norwegian_Sea' or
  #        'Okhotsk' or 'OpenOcean' or 'Pacific_Ocean' or 'Ross_Sea' or 
  #        'Serreze_Arctic' or 'Southern_Hemisphere' or 'StLawr' or 'Subpolar_Gyre'
  #        'TotalArc' or 'Vilkitsky_Strait' or 'Weddell_Sea' or
  #        'Western_Central_Arctic' ], 'moc_40N55N_1-2km',
  #        'moc_30N40N_1-2km', 'max_moc_38N50N_500m-2km', 'max_moc_40N',
  #        'heatc', '0-315_heatc', '373-657_heatc', '800-5350_heatc',
  #        'mxl_heatc', 'NAtl_10N65N_heatc', 'NAtl_10N65N_0-315_heatc',
  #        'NAtl_10N65N_373-657_heatc', 'NAtl_10N65N_800-5350_heatc',
  #        'mxl_NAtl_10N65N_heatc', [ and similarly ('mxl_' followed by 
  #         or '_0-315' or '_373-657' or '_800-5350' preceded by, optional)
  #        'TAtl_30S30N' or 'NPac_10N70N' or 'TPac_30S30N' or 'Arc_65N90N' or 
  #         'Ant_90S60S' or 'TInd_30S30N' followed by 'heatc'.
  #   exp: IMPORTANT: Place first the experiment with the largest number of 
  #        members and, if possible, with the largest number of leadtimes. If
  #        not possible, it is mandatory to specify the argument nleadtime
  #        and/or nmember.
  #        c('EnsEcmwfDec', 'EnsUkmoDec', 'EnsCerfacsDec', 'EnsIfmDec',
  #          'EnsEcmwfSeas', 'EnsCmccSeas', 'EnsIfmSeas', 'EnsMetfrSeas',
  #          'EnsUkmoSeas', 'DePreSysAsimDec', 'DePreSysNoAsimDec',
  #          'DePreSysAsimSeas', 'ECMWF_S3Seas', 'ECMWF_S4_sea',
  #          'ECMWF_S4_ann', 'hadcm3dec', 'miroc4dec', 'miroc5dec',
  #          'mri-cgcm3dec', 'cancm4dec1', 'cancm4dec2', 'cnrm-cm5dec',
  #          'knmidec', 'smhidec', 'mpimdec', 'gfdldec', 'cmcc-cmdec',
  #          'ipsldec', 'bccdec', 'gfdlhis', 'ipslhis', 'cmcc-cmhis', 'bcchis',
  #          'i00k', 'b013', 'b014', ...)
  #   obs: c('ERA40', 'NCEP', 'ERAint', '20thCv2', 'GHCN', 'GHCNERSSTGISS', 'ERSST',
  #          'HadISST', 'GPCP', 'GPCC', 'CRU', 'HadSLP', 'NSIDC', 'PIOMAS',
  #          'UCL', 'DS94', 'OAFlux', 'DFS4.3', 'NCDCglo', 'NCDCland', 
  #          'NCDCoc', 'GISSglo', 'GISSland', 'GISSoc', 'HadCRUT3glo', 
  #          'HadCRUT4', 'HadSST2oc', 'CRUTEM3land')
  #   sdates: c('YYYYMMDD','YYYYMMDD')
  #   lonmin: >= 0, default: 0
  #   lonmax: <= 360, default: 360
  #   latmin: >= -90, default: -90
  #   latmax: <= 90, default: 90
  #   nleadtime: Optional argument needed only if the first experiment in the
  #              parameter exp does not have the largest number of leadtimes.
  #              Default: number of leadtimes of the first experiment.
  #   nmember: Some experiments have more members in starting dates other than
  #            the first. If it is the case in the first experiment specified,
  #            fill nmember with the largest number of members.
  #   leadtimemin: Load into the matrices only the leadtimes from leadtimemin. 
  #                Default: 1.
  #   leadtimemax: Load into the matrices only the leadtimes before leadtimemax.
  #                Default: nleadtime.
  #   storefreq: Frequency at which the data to be loaded are stored in the 
  #              repository. Can take values 'monthly' or 'daily'. 
  #              Default: 'monthly'.
  #   sampleperiod: To load only a subset between leadtimemin and leadtimemax 
  #                 with a specified period of subsampling. 
  #                 Default: 1 (all leadtimes are loaded).
  #   output: 'areave' / 'lon' / 'lat' / 'lonlat'.
  #           1) Time series of area-averaged variables over the specified 
  #              domain.
  #           2) Time series of meridional averages as a function of longitudes.
  #           3) Time series of zonal averages as a function of latitudes.
  #           4) Time series of 2d fields.
  #           Default: 'areave'
  #   method: 'bilinear' / 'bicubic' / 'conservative' / 'distance-weighted'
  #           Method of interpolation for 'lon' / 'lat' / 'lonlat' output 
  #           options.
  #           Default: 'conservative'.
  #   grid: To choose a common output grid.
  #         Possible options: rNXxNY or tTRgrid, ex: r96x72, t106grid.
  #         Default: By default, the data of each experimental dataset will be 
  #         loaded into the matrix using the grid in which the data is stored 
  #         in the repository.
  #         IMPORTANT: If various experiments in different grids are specified, 
  #         a common grid has to be specified using this argument.
  #
  #         How grids work:
  #  
  #         We may load one or more experiments and corresponding data from one 
  #         or more observations.
  #         The most usual is that they are on different grids.
  #
  #         We can set Load() to output the data in 4 different modes:
  #         
  #         If output = 'areave' all the data is area-averaged.
  #         If output = 'lon', the data are averaged along the latitude 
  #         dimension and output as a function of the longitudes.
  #         If output = 'lat', the data are averaged along the longitude 
  #         dimension and output as a function of the latitudes.
  #         If output = 'lonlat', the data are output as a function of the 
  #         longitudes and latitudes.
  #         
  #         We refer to the last 3 modes as maps.
  #
  #         If we want to get data output in maps but the experiments and
  #         observations are on different grids, we need to specify a common 
  #         gridding (with the  parameter grid) onto which the loaded data 
  #         will be interpolated.
  #         If we don't do, Load() will crash.
  #         
  #         If we want to get area-averages, the experiments and observations 
  #         can be on different grids and there is no need to specify a common 
  #         grid: Load() will calculate their mean over the original grids 
  #         unless a common grid is specified.
  #
  #         Whenever we specify a common gridding, all data (observational and
  #         experimental) is interpolated onto it.
  #
  #   maskmod: list(mask[lon,lat]) = 1/0: kept/removed grid cell over the 
  #            entire model domains.
  #            Warning: list() compulsory even if 1 model!!!
  #            Default: 1 everywhere.
  #
  #            How masks work:
  #            
  #            When we want to disable certain values we can specify masks. The 
  #            disabled values will be replaced with NA values.
  #
  #            In the case that you wish to load data as area-averages:
  #            You can specify a mask for each experiment and observation.
  #            Each area average will be performed on the original experiment 
  #            or observation grid and after applying the respective mask.
  #
  #            In the case that you wish to load maps:
  #            You will have defined a mandatory common grid which all the data 
  #            will be interpolated onto.
  #            If you want to specify a mask, you will have to provide it 
  #            already interpolated onto the common grid (you may use 'cdo' for 
  #            this purpose).
  #            It is not usual to apply different masks on experimental 
  #            datasets on the same grid. All the experiment masks are expected 
  #            to be the same.
  #
  #            Defining masks for the observational data will be useless: the 
  #            same mask applied to the experimental data will be applied to
  #            the observational data because the observations are interpolated 
  #            on the common grid for which the mask has already been defined 
  #            when loading experimental data.
  #
  #   maskobs: list(mask[lon,lat]) = 1/0: kept/removed grid cell over the 
  #            entire observed domains, only necessary for 'areave' output 
  #            option.
  #            Warning: list() compulsory even if 1 dataset !!!
  #            Default: 1 everywhere.
  # 
  # Returns:
  #   $mod: Model outputs.
  #         If output = 'areave', matrix with dimensions
  #           c(nmod/nexp or nobs, nmemb/nparam, nsdates, nltime)
  #         If output = 'lat', matrix with dimensions
  #           c(nmod/nexp or nobs, nmemb/nparam, nsdates, nltime, nlat)
  #         If output = 'lon', matrix with dimensions
  #           c(nmod/nexp or nobs, nmemb/nparam, nsdates, nltime, nlon)
  #         If output = 'lonlat', matrix with dimensions
  #           c(nmod/nexp or nobs, nmemb/nparam, nsdates, nltime, nlat, nlon)
  #   $obs: Observations. Matrix with same dimensions as '$mod'.
  #   $lat: Latitudes of the model grid.
  #   $lon: Longitudes of the model grid.
  # 
  # Details:
  #
  #   Load() is repository-specific, that meaning you should overwrite it so as 
  #   to make it work properly with the directory tree and naming conventions 
  #   in your data repository.
  #   The current layout and documentation is the one being used at IC3 by way 
  #   of illustration.
  #
  #   The function Load() must return a list which contains four components: 
  #   $mod, $obs, $lat and $lon.
  #   $mod is the matrix containing the experimental data.
  #   $obs is the matrix containing the observational data.
  #   $lat and $lon are the latitudes and longitudes of the model grid (0 if 
  #   the loaded variable is a global mean).
  #
  #   The experimental data matrix will contain the values of a given variable 
  #   for all the experiments the user wish to compare for a set of starting 
  #   dates, lead-times, longitudes and latitudes interpolated to the common 
  #   grid and with the masks applied.
  #   The observational data matrix will contain the observed values of the 
  #   same variable gathered from the specified observational datasets scanning 
  #   the data repository. The observational data is chosen so as to 
  #   date-correspond the experimental data.
  #
  #   The data matrices must be built with the dimensions in the following 
  #   order and with the following lengths:
  #   1- The number of experimental datasets determined by the user (we need to 
  #   store data of all the experiments in the same array) for the experimental 
  #   matrix or the number of observational datasets available for validation 
  #   in the observational matrix.
  #   2- The greatest number of members across all experiments.
  #   3- The number of starting dates determined by the user (we need to store 
  #   data of each prediction of the model in each starting date).
  #   4- The greatest number of lead-times.
  #   5- The number of latitudes of the zone we want to consider.
  #   6- The number of longitudes of the zone we want to consider.
  #
  #   Dimensions 5 and 6 are optional and their presence will depend on 
  #   whether the variable loaded is 2-dimensional or an area average. 
  #   In the case of an area average the dimensions of the matrix will be only 
  #   the first 4.
  #
  #   For example, at IC3, we have implemented Load() to select the 
  #   experimental and observational datasets from which to load data, the 
  #   variable to load, the range of starting dates, the range of lead-times, 
  #   the frequency and step in which the data is loaded from the datasets, a 
  #   common grid, a method of interpolation, a mask to disable certain 
  #   experimental values, a mask to disable certain observational values and 
  #   an output format:
  #     1) Time series of area-averaged variables over the specified domain. 
  #     2) Time series of meridional averages as a function of longitudes. 
  #     3) Time series of zonal averages as a function of latitudes.
  #     4) Time series of 2d fields.
  #
  #   Furthermore, if the user loads a 2-dimensional variable, he/she can ask 
  #   Load() to load it as area averages, as longitudinal averages in function 
  #   of latitudes or as in latitudinal averages in function of longitudes. In 
  #   these cases the 5th and/or 6th dimension will also disappear.
  #
  #   The observational data matrix is built very similarly to the experimental 
  #   matrix. The only differences are that the first dimension turns to be the
  #   number of observational datasets and the length of the 2nd dimension 
  #   turns to be the number of members of the observational datasets. Later, 
  #   for each lead-time we will store observational data into it if possible.
  #
  #   Any value that can't be filled (either in the experimental matrix or the 
  #   observational matrix) should figure as an NA value.
  #
  #   
  #   This process can become time-expensive when many experiments or 
  #   observational datasets are involved.
  #   We suggest to use the package 'parallel' to minimize the time by 
  #   performing this task in multiple threads of execution.
  #
  #   This function will very probably require to use the 'ncdf' package.
  #
  # History:
  #   1.0  #  2011-03  (V. Guemas, virginie.guemas@ic3.cat)     # Original code
  #   1.1  #  2014-03  (N. Manubens, nicolau.manubens@ic3.cat)  # Formatting to CRAN
  
  ## Packages and resources required.
  #library(ncdf)

  # Extend the current environment with the functions and variables related to 
  # the configuration file mechanism which contain the information about the 
  # location of the datasets.
  atm_vars <- oceanic_vars <- experiments <- file_per_ensemble_exps <- NULL
  observations <- putInDictionary <- getDatasetInfo <- NULL
  replaceVariablesInString <- NULL
  LoadConfigurationFile(environment())
  
  # Check that the parameters are right
  has_entries_in_atm_vars <- sum(apply(array(atm_vars), 1, regexpr, var) > 0) > 0
  has_entries_in_oceanic_vars <- sum(apply(array(oceanic_vars), 1, regexpr, var) > 0) > 0
  if (!has_entries_in_atm_vars && !has_entries_in_oceanic_vars) {
    print('The variable specified is not available. Please check the supported variables in the configuration file.')
  }
  
  has_entries_in_exps <- c(FALSE, length(exp))
  has_entries_in_file_per_ensemble_exps <- c(FALSE, length(exp))
  for (jmod in 1:length(exp)) {
    has_entries_in_exps[jmod] <- sum(apply(array(unlist(lapply(experiments, lapply, "[[", 1))), 1, regexpr, exp[jmod]) > 0) > 0
    has_entries_in_file_per_ensemble_exps[jmod] <- sum(apply(array(unlist(lapply(file_per_ensemble_exps, lapply, "[[", 1))), 1, regexpr, exp[jmod]) > 0) > 0
  }
  if (sum(has_entries_in_exps | has_entries_in_file_per_ensemble_exps) < length(exp)) {
    print('The following experiments are not available:')
    exp[which((has_entries_in_exps | has_entries_in_file_per_ensemble_exps) == FALSE)]
    print('Please check the supported experiments in the configuration file.')
  }

  has_entries_in_obs <- c(FALSE, length(obs))
  for (jobs in 1:length(obs)) {
    has_entries_in_obs[jobs] <- sum(apply(array(unlist(lapply(observations, lapply, "[[", 1))), 1, regexpr, obs[jobs]) > 0) > 0
  }
  if (sum(has_entries_in_obs) < length(obs)) {
    print('The following observations are not available:')
    obs[which(has_entries_in_obs == FALSE)]
    print('Please check the supported experiments in the configuration file.')
  }
 
  # 
  #  Get matrices dimensions from the input
  # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  #
  # The two data matrices are similar and have 7 dimensions each with the 
  # following lengths:
  #   1) Number of experimental/observational datasets.
  #   2) Number of members.
  #   3) Number of starting dates.
  #   4) Number of lead-times.
  #   5) Number of latitudes (optional).
  #   6) Number of longitudes (optional).
  # 
  # We can give values to some of these lengths already. 
  # The lengths will vary in function of the datasets we load and will be 
  # re-adjusted along the process.
  #
  ## nmod is the number of experimental datasets in the repository we load data 
  ## from, and will be the length of the first dimension of the experimental 
  ## data matrix.
  nmod <- length(exp)
  ## nmemb is the largest number of members among the experiments. Initially it 
  ## is set to 9 because all DePreSys experiments have 9 members. If it is 
  ## another kind of experiment, the number of members is adjusted later.
  nmemb <- 9
  ## nobs is the number of observational datasets we load data from.
  nobs <- length(obs)
  ## nsdates is the number of starting dates specified by the user. Only data 
  ## of simulations started on these dates is loaded into the experimental data 
  ## matrix. 
  nsdates <- length(sdates)
  
  ## The following variables are state flags. The questions commented above 
  ## each explain their meaning.
  ## Are the dimensions of the matrices yet defined?
  dims2define <- TRUE
  ## Are the matrices yet created?
  mat2define <- TRUE
  ## Do we have to arrange the loaded data with the default grid of each  
  ## experiment? If not, the data is arranged with a common grid defined by the 
  ## user using 'cdo' tools.
  defgrid <- is.null(grid)
  ## Is the mask for each experiment still to be created?
  flag_mask_mod <- array(TRUE, dim = nmod)
  ## Is the mask for each observation still to be created?
  flag_mask_obs <- array(TRUE, dim = nobs)
  ## We choose the kind of interpolation for the 'cdo regrid' tool.
  remap <- switch(method, 'bilinear' = 'remapbil', 'bicubic' = 'remapbic', 
                  'conservative' = 'remapcon', 'distance-weighted' = 'remapdis')
  
  ## For each experiment we will keep a matrix of weights to make data in wider
  ## areas more important.
  ## Due to NA values in the data the weights will change among experiments, 
  ## thus it is needed to keep a matrix of weights for each experiment.
  weight_area <- list()
  ## bugfix: weight_lon <- list()
  ## For each observational dataset we keep likely a matrix of weights.
  weight_area_obs <- list()
  ## bugfix: weight_lon_obs <- list()
  ## lat_mod and lon_mod will be set later with the latitudes and longitudes of 
  ## the common grid applied to the data of the experiments and will be  
  ## returned as outputs by Load().
  lat_mod <- NULL
  lon_mod <- NULL
  
  ## Make sure that longitude bounds are positive
  if  (lonmin < 0) {
    lonmin <- lonmin + 360
  }
  if (lonmax < 0) {
    lonmax <- lonmax + 360
  }

  ## If the chosen variable is atmospheric, we can already work out a part of 
  ## the path of the dataset (freq). Also we know that it is a 2-dimensional 
  ## variable and that the observational datasets for this variable will have 
  ## only one member.
  is_atm_var <- has_entries_in_atm_vars
  ## The observational datasets have only one member except ORAS4 ocean 
  ## reanalysis.
  nmembobs <- 1
  if (!is_atm_var) {
  ## If the variable is oceanic, we always use the default grid. We know that  
  ## it is not a 2-dimensional variable and that the data loaded will be area 
  ## averages mandatory and cannot be converted.
    defgrid <- TRUE
    output <- 'areave'
  }

  # Now we are ready to start loading data into the matrices.
  # We will loop on the specified starting dates. 
  #   For each starting date, we will loop on the list of experimental datasets. 
  #     For each experimental dataset, we will load the file that contains the 
  #     data of all of its members in the current starting date and then we 
  #     will loop on the members.
  #       For each member, we will apply the weights, mask and the appropriate 
  #       transformations and finally store it into the experimental data 
  #       matrix.
  #       However, if the experiment is 'DePreSys', the data of each member in 
  #       the repository is stored in a separate file. In that case the file  
  #       for each member is read at the time we loop on the members instead of 
  #       once at the time we loop on the experimental dataset.
  #   After we have the experimental data for the current starting date, we  
  #   load the observational data from the specified repositories that date 
  #   correspond the lead-times of the specified experimental datasets in the 
  #   current starting date.
  
  ## indyear determines how many starting dates we have looped on so far.
  indyear <- 1
  for (jyear in sdates) {  # Loop on starting dates
    # Put the starting date in the table of variables for the configuration file.
    replace_values <- putInDictionary(replace_values, "START_DATE", jyear)
    #
    #  Loading experimental data
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # 
    for (jmod in 1:nmod) {  # Loop on experimental datasets
      # Put the experiment name in the table of variables for the configuration file.
      replace_values <- putInDictionary(replace_values, "EXP_NAME", exp[jmod])
      ## If the current experimental dataset is not 'DePreSys' then we load the
      ## data for all of its members in the current starting date from a single 
      ## file.
      if (!has_entries_in_file_per_ensemble_exps[jmod] && has_entries_in_exps[jmod]) {
        # Get the information of the experiment
        dataset_info <- getDatasetInfo(experiments, exp[jmod], var)
        # and put its associated values in the dictionary
        replace_values <- putInDictionary(replace_values, "EXP_MAIN_PATH", dataset_info[1])
        replace_values <- putInDictionary(replace_values, "EXP_FILE_PATH", dataset_info[2])
        replace_values <- putInDictionary(replace_values, "GRID", dataset_info[3])
        replace_values <- putInDictionary(replace_values, "NC_VAR_NAME", dataset_info[4])
        namevar <- replaceVariablesInString(dataset_info[4], replace_values)
        ## In function of the name of the variable and the dataset and other
        ## parameters we build the path to the file, following the rules 
        ## specified in the configuration file.
        filename <- replaceVariablesInString("$EXP_FULL_PATH$", replace_values)
        ## We obtain the names of all data files in the directory of the 
        ## repository.
        filename
        argfiles <- paste('ls', filename)
        files <- system(argfiles, intern = TRUE, ignore.stderr = TRUE)
        
        ## If we do not find any file there, we print a warning and set as file
        ## name 'nofile.nc'. If we find any, we take the last as file name.
        ## Then we set the grid. If the grid to apply is the default grid of  
        ## the experiment we do not apply changes to the data file, so the file 
        ## name remains the same.
        ## If the grid has to be changed, we use 'cdo' for this purpose and 
        ## then save the changes into a temporary file named 'tmp.nc'.
        if (length(files) > 0) {
          filename <- files[length(files)]
          print(filename)        
          if (defgrid) {
            grid <- replaceVariablesInString(dataset_info[3], replace_values)
            filein <- filename
          } else {
            system(paste("cdo ", remap, ",", grid, " -selname,", namevar, " ", filename, 
                         " tmp.nc 2>/dev/null", sep = ""))
            system("ncrename -v lat,latitude -d lat,latitude -v lon,longitude -d lon,longitude tmp.nc")
            filein <- "tmp.nc"
          }
          
          ## We open the file whichever is its name and we load into the 
          ## variable tmp the data of the specified variable it contains.
          fnc <- open.ncdf(filein)
          tmp <- get.var.ncdf(fnc, namevar)
          
          ## Now we have to arrange the data in the matrix tmp with the 
          ## dimensions in a common order:
          ##   1) Longitudes 
          ##   2) Latitudes
          ##   3) Members
          ##   4) Lead-times
          ##
          ## If the variable is 2-dimensional (atmospheric):
          if (is_atm_var) {
            ## We disable large values.
            tmp[which(tmp > 1e11)] <- NA
            tmp[which(tmp < (-1e33))] <- NA
            ## If the matrix loaded has only 3 dimensions, it means it only 
            ## contains data for one member so the dimension of the members has
            ## length 1 and has been automatically squeezed.
            ## It has to be added.
            if (length(dim(tmp)) == 3) {
              tmp <- InsertDim(tmp, 3, 1)
            }
            ## We read the longitudes and latitudes from the file.
            lat <- get.var.ncdf(fnc, "latitude")
            lon <- get.var.ncdf(fnc, "longitude")
            ## We re-order the latitudes if needed.
            if (lat[1] < lat[length(lat)]) {         
              tmp <- tmp[, length(lat):1, , ]
              lat <- lat[length(lat):1]
              if (length(dim(tmp)) == 3) {
                tmp <- InsertDim(tmp, 3, 1)
              }
            }
          ## If the variable is a global mean (oceanic):
          } else {
            ## In this case, the file should contain data in two dimensions: 
            ## members and lead-times.
            ## If any of them does not appear, it is because there was only one 
            ## member or only one lead-time and was automatically squeezed.
            if (is.null(dim(tmp))) {
              tmp <- InsertDim(tmp, 1, 1)
            }
            if (length(dim(tmp)) < 2) {
              tmp <- InsertDim(tmp, 1, 1)
            }
            ## We add two dimensions corresponding to latitudes and longitudes 
            ## to fulfill the common order.
            tmp <- InsertDim(tmp, 1, 1)
            tmp <- InsertDim(tmp, 1, 1)
            ## We set as output longitudes and latitudes one single point 
            ## (0, 0) to avoid a crash.
            lat_mod <- 0
            lon_mod <- 0
          }
          close.ncdf(fnc)
          ## At this point, if we are in the first experiment, we can define 
          ## the definitive lengths of the dimensions of the experimental 
          ## matrix.
          if (dims2define) {
            if (is.null(nleadtime)) {
              nleadtime <- dim(tmp)[4]
            }
            if (is.null(leadtimemax)) {
              leadtimemax <- nleadtime
            }
            if (is.null(nmember)) {
              nmemb <- dim(tmp)[3]
            } else {
              nmemb <- nmember
            } 
            dims2define <- FALSE
          }
          ## If we are not in the first experiment, we check that the length of 
          ## the dimensions of data of the current experiment does not exceed 
          ## the lengths of the experimental matrix to make sure no data is 
          ## lost.
          else {
            if (dim(tmp)[3] > nmemb) {
              stop("PUT FIRST THE EXPERIMENT ID WITH THE LARGEST NUMBER OF MEMBERS.
                    If already done, check the dimensions of your files in the repository.")
            }
            if (dim(tmp)[4] > nleadtime) {
              stop("nleadtime argument is wrong")
            }
          }
        } else {
          print(paste('No file in ', argfiles))
          filename <- 'nofile.nc'
        }
      }
      
      ## Now, if the experiment was not 'DePreSys', the data for all the 
      ## members in the current starting date is already loaded into the matrix 
      ## tmp.
      # We are ready to loop on the members.
      for (jmemb in 1:nmemb) {
        #Put the ensemble number in the table of variables for the configuration file.
        replace_values <- putInDictionary(replace_values, "ENSEMBLE_NUMBER", jmemb)
        # If the experiment is 'DePreSys', we have to do the same process to 
        # load each file that contains data for the current member.
        if (has_entries_in_file_per_ensemble_exps[jmod]) {
          # Get the information of the experiment
          dataset_info <- getDatasetInfo(file_per_ensemble_exps, exp[jmod], var)
          # and put its associated values in the dictionary
          replace_values <- putInDictionary(replace_values, "EXP_MAIN_PATH", dataset_info[1])
          replace_values <- putInDictionary(replace_values, "EXP_FILE_PATH", dataset_info[2])
          replace_values <- putInDictionary(replace_values, "GRID", dataset_info[3])
          replace_values <- putInDictionary(replace_values, "NC_VAR_NAME", dataset_info[4])
          namevar <- replaceVariablesInString(dataset_info[4], replace_values)

          ## We build the path to the dataset.
          filename <- replaceVariablesInString("$EXP_FULL_PATH$", replace_values)
          print(filename)
          ## If the file is found, we put the data in the corresponding grid 
          ## and store it into the matrix tmp. We disable large values and 
          ## re-order the latitudes if needed.
          ## The data loaded will correspond to a single starting date of a 
          ## single experiment and to a single member. So it will have 3 
          ## dimensions:
          ##   1) Longitudes
          ##   2) Latitudes
          ##   3) Lead-times
          if (file.exists(filename)) {
            if (defgrid) {
              grid <- replaceVariablesInString(dataset_info[3], replace_values)
              filein <- filename
            } else {
              system(paste("cdo ", remap, ",", grid, " -selname,", namevar, " ", filename, 
                           " tmp.nc 2>/dev/null", sep = ""))
              system("ncrename -v lat,latitude -d lat,latitude -v lon,longitude -d lon,longitude tmp.nc")
              filein <- "tmp.nc"
            }
            fnc <- open.ncdf(filein)
            tmp <- get.var.ncdf(fnc, namevar)
            tmp[which(tmp > 1e11)] <- NA
            tmp[which(tmp < (-1e33))] <- NA
            lat <- get.var.ncdf(fnc, "latitude")
            lon <- get.var.ncdf(fnc, "longitude")
            close.ncdf(fnc)
            if (lat[1] < lat[length(lat)]) { 
              tmp <- tmp[, length(lat):1, ]
              lat <- lat[length(lat):1]
            }
            
            if (dims2define) {
              if (is.null(nleadtime)) {
                nleadtime <- dim(tmp)[3]
              }
              if (is.null(leadtimemax)) {
                leadtimemax <- nleadtime
              }
              dims2define <- FALSE
            }
          }
        }
        
        ## At this point, if the experiment is 'DePreSys', we have data of the 
        ## current member in the matrix tmp (with 3 dimensions). 
        ## If the experiment is not 'DePreSys', we have data of all of the 
        ## members (4 dimensions).
        ##
        # If data was found in the repository, we apply the mask and weights to 
        # it and store into the experimental data matrix.
        # If no data was found, the output matrix will contain NA values for 
        # the current member (DePreSys) or starting date (non DePreSys) of the 
        # current experiment.
        ## jens <- (jmod - 1) * nmemb + jmemb
        if (file.exists(filename)) {
          ## If the variable is 2-dimensional:
          if (is_atm_var) {
            # We loop on the lead-times. 
            for (jtime in seq(leadtimemin, leadtimemax, sampleperiod)) {
              ## For each lead-time we create a matrix
              ## of size longitudes x latitudes and fill it in with the data in 
              ## the matrix tmp that corresponds to the current lead-time.
              tmp1 <- array(dim = c(length(lon), length(lat)))
              if (has_entries_in_file_per_ensemble_exps[jmod]) {
                if (jtime <= dim(tmp)[3]) {
                  tmp1 <- tmp[, , jtime]
                }
              } else {
                if (jmemb <= dim(tmp)[3] & jtime <= dim(tmp)[4]) {
                  tmp1 <- tmp[, , jmemb, jtime]
                }
              }
              #
              #  Selecting the masked domain
              # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
              #
              indlon <- which((lon >= lonmin & lon <= lonmax) | 
                              (lonmax < lonmin & (lon >= lonmin | lon <= lonmax)
                              ))
              tmp2 <- tmp1[indlon, which(lat >= latmin & lat <= latmax)]
              ## In tmp2, we have the data of the current lead-time placed
              ## inside the specified longitudes and latitudes.
              ## If we have not created the mask of te current model yet:
              if (flag_mask_mod[jmod]) {
                ## If the user did not specify a mask, we create one full of 1s.
                if (is.null(maskmod[[jmod]])) {
                  maskmod[[jmod]] <- array(1, dim = dim(tmp1))
                }
                ## Else, we take the specified mask.
                ## We slice it to fit the limited area.
                maskmod[[jmod]] <- maskmod[[jmod]][indlon, 
                                   which(lat >= latmin & lat <= latmax)]
                ## We define the output longitudes and latitudes
                lat_mod <- lat[which(lat >= latmin & lat <= latmax)]
                lon_mod <- lon[indlon]
                ## Now the mask is marked as created.
                flag_mask_mod[jmod] <- FALSE
                
                #
                #  Computing weigths for averaging
                # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                #
                ## If the mask was not created yet, the weights were not 
                ## computed either. We coumpute them.
                weight <- InsertDim(cos(lat_mod * pi / 180), 1, dim(tmp2)[1])
                weight[which(maskmod[[jmod]] < 0.5 | is.na(tmp2))] <- NA
                weight_lon <- weight / InsertDim(Mean1Dim(weight, 2, narm = TRUE),
                                                 2, dim(tmp2)[2])
                ## bugfix: 
                ##weight_lon[[jmod]] <- weight / InsertDim(Mean1Dim(weight, 2, 
                ##                                                  narm = TRUE),
                ##                                         2, dim(tmp2)[2])
                weight_area[[jmod]] <- weight / mean(weight, na.rm = TRUE)
                
                #
                #  Defining output matrix
                # ~~~~~~~~~~~~~~~~~~~~~~~~
                #
                ## Now we will apply the weights and store the result into the
                ## experimental matrix. If it has not been created, we create 
                ## it.
                ## We also create the observational matrix as it is the same 
                ## size.
                ## If the user specified an output type of area average, there
                ## will not be the 5th and 6th dimensions of longitudes and 
                ## latitudes, and so on.
                if (mat2define) {
                  nleadtime2 <- (leadtimemax - leadtimemin) %/% sampleperiod + 1
                  var_exp <- switch(output,
                                    'areave' = array(dim = c(nmod, nmemb, 
                                               nsdates, nleadtime2)),
                                    'lon' = array(dim = c(nmod, nmemb, nsdates,
                                            nleadtime2, dim(tmp2)[1])),
                                    'lat' = array(dim = c(nmod, nmemb, nsdates,
                                            nleadtime2, dim(tmp2)[2])),
                                    'lonlat' = array(dim = c(nmod, nmemb, 
                                               nsdates, nleadtime2, 
                                               dim(tmp2)[2:1])))
                  var_obs <- switch(output, 
                                    'areave' = array(dim = c(nobs, nmembobs,
                                               nsdates, nleadtime2)),
                                    'lon' = array(dim = c(nobs, 1, nsdates, 
                                            nleadtime2, dim(tmp2)[1])),
                                    'lat' = array(dim = c(nobs, 1, nsdates,
                                            nleadtime2, dim(tmp2)[2])),
                                    'lonlat' = array(dim = c(nobs, 1, nsdates,
                                               nleadtime2, dim(tmp2)[2:1])))
                  ## We mark that the matrices are now created.
                  mat2define <- FALSE
                }
              }
              ## We apply the mask to the data.
              tmp2[which(maskmod[[jmod]] < 0.5)] <- NA
              # 
              #  Computing the outputs
              # ~~~~~~~~~~~~~~~~~~~~~~~
              #
              # We apply the weights to the data and store it into the 
              # experimental matrix.
              jtime2 <- (jtime - leadtimemin) %/% sampleperiod + 1
              if (output == 'areave') {
                var_exp[jmod, jmemb, indyear, jtime2] <- mean(
                                                         tmp2 * weight_area[[jmod]], 
                                                         na.rm = TRUE)
              } else if (output == 'lon') {
                var_exp[jmod, jmemb, indyear, jtime2, ] <- Mean1Dim(
                                                           tmp2 * weight_lon,
                                                           2, narm = TRUE)
                ## bugfix:
                ##                                           weight_lon[[jmod]],
              } else if (output == 'lat') {
                var_exp[jmod, jmemb, indyear, jtime2, ] <- Mean1Dim(tmp2, 1,
                                                           narm = TRUE)
              } else if (output == 'lonlat') {
                var_exp[jmod, jmemb, indyear, jtime2, , ] <- t(signif(tmp2, 5))
              } else {
                stop("Wrong output option")
              }
            }
          ## If the variable is a global mean:
          } else {
            ## We do not need to apply masks or weights. Just create the 
            ## matrices if not yet created and store the data into the 
            ## experimental one.
            if (mat2define) {
              nleadtime2 <- (leadtimemax - leadtimemin) %/% sampleperiod + 1
              var_exp <- array(dim = c(nmod, nmemb, nsdates, nleadtime2))
              var_obs <- array(dim = c(nobs, nmembobs, nsdates, nleadtime2))
              mat2define <- FALSE
            }
            if (dim(tmp)[3] >= jmemb) {
              var_exp[jmod, jmemb, indyear, 1:min(nleadtime2, 
                (dim(tmp)[4] - leadtimemin) %/% sampleperiod + 1)] <- tmp[1, 1, 
                jmemb, seq(leadtimemin, min(leadtimemax, dim(tmp)[4]), 
                sampleperiod)]
            }
          }
        }
      }
    }
  
    #
    #  Loading observational data
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    #
    # Now we have to load the observational data in the current starting date
    # from the specified observational datasets that date corresponds the 
    # experimental data.
    # Each file in the repository contains data of one month.
    
    ## indobs tells which observational dataset we are looping on.
    indobs <- 1
    for (jobs in obs) {  # Loop on observational datasets.
      # Put the observation name in the table of variables for the configuration file.
      replace_values <- putInDictionary(replace_values, "OBS_NAME", jobs)
      # Get the information of the experiment
      dataset_info <- getDatasetInfo(observations, jobs, var)
      # and put its associated values in the dictionary
      replace_values <- putInDictionary(replace_values, "OBS_MAIN_PATH", dataset_info[1])
      replace_values <- putInDictionary(replace_values, "OBS_FILE_PATH", dataset_info[2])
      replace_values <- putInDictionary(replace_values, "NC_VAR_NAME", dataset_info[3])
      namevar <- replaceVariablesInString(dataset_info[3], replace_values)
      
      ## If the variable is atmospheric:
      if (is_atm_var) {
        ## The current starting date is transformed into a number.
        if (storefreq == 'daily') {
          iday <- substr(jyear, 7, 8)
          if (iday == '') {
            iday <- '01'
          } 
          startdate <- as.POSIXct(paste(substr(jyear, 1, 4), '-',
                       substr(jyear, 5, 6), '-', iday, ' 12:00:00', 
                              sep = '')) + (leadtimemin - 1) * 86400
        } else {
          imon <- (as.integer(substr(jyear, 5, 
                   6)) + leadtimemin - 2) %% 12 + 1
          iyear <- as.integer(substr(jyear, 1, 4)) + (as.integer(substr(jyear, 
                    5, 6)) + leadtimemin - 2) %/% 12
        }
        ## jtime is the index of the current lead-time.
        jtime <- 1
        # We loop on the lead-times.
        while (jtime <= nleadtime2) {
          ## The current lead-time is calculated from the current starting 
          ## date as number and from the index of the current lead-time.
          if (storefreq == 'daily') {
            newdate <- startdate + 86400 * (jtime - 1) * sampleperiod
            iyear <- substr(newdate, 1, 4)
            imon <- as.integer(substr(newdate, 6, 7))
            itimind <- as.integer(substr(newdate, 9, 10))
            if (LeapYear(as.integer(iyear))) {
              lenmon <- 29
            } else { 
              lenmon <- 28
            }
            lenmon <- switch(substr(newdate, 6, 7), '01' = 31, '03' = 31, 
                      '04' = 30, '05' = 31, '06' = 30, '07' = 31, '08' = 31,
                      '09' = 30, '10' = 31, '11' = 30, '12' = 31, lenmon)
          } else {
            if (jtime > 1) {
              imon <- imon + sampleperiod
              iyear <- iyear + (imon - 1) %/% 12
              imon <- (imon - 1) %% 12 + 1
            }
            itimind <- 1
            lenmon <- 1
          }
          ## Put the current year and month in the dictionary of values.
          replace_values <- putInDictionary(replace_values, "YEAR", iyear)
          replace_values <- putInDictionary(replace_values, "MONTH", sprintf("%2.2i", imon))
          ## The full path to the file that corresponds to the current 
          ## lead-time is built.
          filename <- replaceVariablesInString("$OBS_FULL_PATH$", replace_values)
          # If the file is found, the data is loaded into the observational 
          # matrix after putting in order and applying the mask and weights.
          # If not, the associated part of the matrix is left with NA values.
          if (file.exists(filename)) {
            print(filename)             
            if (output != 'areave' | defgrid == FALSE) {
              system(paste("cdo ", remap, ",", grid, " -selname,", namevar, " ", filename, 
                           " tmp.nc 2>/dev/null", sep = ""))
              system("ncrename -v lat,latitude -d lat,latitude -v lon,longitude -d lon,longitude tmp.nc")
              filename <- "tmp.nc"
            }
            fnc <- open.ncdf(filename)
            tmp <- get.var.ncdf(fnc, namevar)
            # TODO:
            if (var == 'tos' | var =='tas' | var == 'prlr') { 
              tmp[which(tmp > 599 | tmp < (-999))] <- NA 
            }
            # TODO:
            if (substr(jobs, (nchar(jobs) - 3), 
                nchar(jobs)) != 'land' & substr(jobs, (nchar(jobs) - 1), 
                nchar(jobs)) != 'oc' & substr(jobs, (nchar(jobs) - 2), 
                nchar(jobs)) != 'glo') {
              lat <- get.var.ncdf(fnc, "latitude")
              lon <- get.var.ncdf(fnc, "longitude")
              lon[which(lon < 0)] <- lon[which(lon < 0)] + 360
            }
            close.ncdf(fnc)
            if (length(dim(tmp)) == 2) {
              tmp <- InsertDim(tmp, 3, 1)
            }
            if ((output != 'areave' | defgrid == FALSE) & lat[1] < 0) { 
              tmp <- tmp[, length(lat):1, ] 
              lat <- lat[length(lat):1]
            }
            if (length(dim(tmp)) == 2 ) {
              tmp <- InsertDim(tmp, 3, 1)
            }
            #
            #  Selecting the masked domain 
            # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            #
            if (storefreq == 'monthly') {
              rangeind <- jtime
            } else {
              rangeind <- jtime:min((lenmon - itimind
                                     ) %/% sampleperiod + jtime, nleadtime2)
            }
            rangeindout <- seq(itimind, min(lenmon, (nleadtime2 - jtime + 1
                               ) * sampleperiod + itimind - 1), sampleperiod)
            if (substr(jobs, (nchar(jobs) - 3), 
                nchar(jobs)) != 'land' & substr(jobs, (nchar(jobs) - 1), 
                nchar(jobs)) != 'oc' & substr(jobs, (nchar(jobs) - 2), 
                nchar(jobs)) != 'glo') {
              indlon <- which((lon >= lonmin & lon <= lonmax) | (
                              lonmax < lonmin & (lon >= lonmin | lon <= lonmax
                              )))
              tmp1 <- tmp[indlon, which(lat >= latmin & lat <= latmax), ]
              if (length(dim(tmp1)) == 2 ) {
                tmp1 <- InsertDim(tmp1, 3, 1) 
              }
              lat1 <- lat[which(lat >= latmin & lat <= latmax)]
              if (flag_mask_obs[indobs]) {
                if (output != 'areave' | defgrid == FALSE) {
                  maskobs[[indobs]] <- maskmod[[nmod]] 
                } else {
                  if (is.null(maskobs[[indobs]])) {
                    maskobs[[indobs]] <- array(1, dim = dim(tmp)[1:2])
                  } 
                  maskobs[[indobs]] <- maskobs[[indobs]][indlon, 
                                       which(lat >= latmin & lat <= latmax)] 
                }
                flag_mask_obs[indobs] <- FALSE
                #
                #  Computing weigths for averaging
                # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                #
                weight <- InsertDim(cos(lat1 * pi / 180), 1, dim(tmp1)[1])
                weight[which(maskobs[[indobs]] < 0.5 | is.na(tmp1[, , 
                       1]))] <- NA
                ## bugfix: weight_lon_obs[[jobs]] <- weight / InsertDim(Mean1Dim(weight, 
                ##                          2, narm = TRUE), 2, dim(tmp1)[2])
                weight_area_obs[[jobs]] <- weight / mean(weight, na.rm = TRUE)
              }
              tmp1[which(InsertDim(maskobs[[indobs]], 3, lenmon) < 0.5)] <- NA
              #
              #  Computing the outputs
              # ~~~~~~~~~~~~~~~~~~~~~~~
              #
              if (output == 'areave') {
                for (ind in 1:lenmon) { 
                  var_obs[indobs, 1, indyear, rangeind[ind]] <- mean(tmp1[, ,
                  rangeindout[ind]] * weight_area_obs[[jobs]], na.rm = TRUE)
                }
              } else if (output == 'lon') {
                var_obs[indobs, 1, indyear, rangeind, ] <- aperm(Mean1Dim(
                  tmp1 * InsertDim(weight_lon, 3, lenmon), 2,
                  narm = TRUE)[, rangeindout], length(dim(tmp1[, , 
                  rangeindout])):1)
                ## bugfix:
                ## * InsertDim(weight_lon_obs[[jobs]], 3, lenmon), 2,
              } else if (output == 'lat') {
                var_obs[indobs, 1, indyear, rangeind, ] <- aperm(Mean1Dim(tmp1
                , 1, narm = TRUE)[, rangeindout], 
                length(dim(tmp1[, , rangeindout])):1)
              } else if (output == 'lonlat') {
                var_obs[indobs, 1, indyear, rangeind, , ] <- aperm(
                signif(tmp1[, , rangeindout], 5), 
                length(dim(tmp1[, , rangeindout])):1)
              } else {
                stop("Wrong output option")
              }
            } else {
              var_obs[indobs, 1, indyear, rangeind] <- tmp[rangeindout]
            }
          }
          jtime <- jtime + (lenmon - itimind) %/% sampleperiod + 1
        }
      ## If the variable is oceanic, it will be a global mean so the weights 
      ## do not need to be calculated:
      } else {      
        ## We build the path to the dataset.
        filein <- replaceVariablesInString("$OBS_FULL_PATH$", replace_values)
        ## Load the values into tmp.
        fnc <- open.ncdf(filein)
        tmp <- get.var.ncdf(fnc, namevar)
        ## Put dimensions in order.
        if (is.na(dim(tmp)[2])) {
          tmp <- InsertDim(tmp, 1, 1)
        }
        close.ncdf(fnc)
        mons <- strsplit(system(paste('cdo showmon ', filein, ' 2>/dev/null'),
                                intern = TRUE), split = ' ')
        years <- strsplit(system(paste('cdo showyear ', filein, ' 2>/dev/null'),
                                 intern = TRUE), split = ' ')
        mons <- as.integer(mons[[1]][which(mons[[1]] != "")])
        years <- as.integer(years[[1]][which(years[[1]] != "")])
        tmp_ts <- ts(t(tmp), start = c(years[1], mons[1]), 
                     end = c(years[length(years)], mons[length(mons)]),
                     frequency = 12)
        tmp2 <- window(tmp_ts, start = c(as.integer(substr(jyear, 1, 4)), 
                       as.integer(substr(jyear, 5, 6))), end = c(3000, 12),
                       frequency = 12, extend = TRUE)
        ## Store values into observational matrix.
        var_obs[indobs, 1:dim(tmp)[1], indyear, ] <- as.matrix(t(tmp2))[, 
                                                     seq(leadtimemin, 
                                                         leadtimemax,
                                                         sampleperiod)]
      }
      ## Loop on the next observational dataset.
      indobs <- indobs + 1
    } 
    ## Loop on the next starting date.
    indyear <- indyear + 1
  }
  ## Remove auxiliar files.
  system("rm -f tmp.nc")
  #
  #  Outputs
  # ~~~~~~~~~
  #
  invisible(list(mod = var_exp, obs = var_obs, lat = lat_mod, lon = lon_mod))
}
